<?php

require_once($CFG->dirroot . '/course/lib.php');

define('FORMAT_QMULTOPICS_IMAGE_HEIGHT', 200);
define('FORMAT_QMULTOPICS_IMAGE_NAME', 'qmt1');

/*
 * Functions for all QMUL news formats
 */


/**
 * Get the image file for the news
 * @param object $context - the course context object
 * @param object $section - the course section object
 * @return mixed stored_file | false
 */
function format_qmultopics_getimagefile($context, $section) {
    $fs = get_file_storage();
    $files = $fs->get_area_files($context->id, 'course', 'section', $section->id);
    foreach ($files as $file) {
        $imagename = FORMAT_QMULTOPICS_IMAGE_NAME.'.';
        if (substr_compare($file->get_filename(), $imagename, 0, strlen($imagename)) == 0) {
            return $file;
        }
    }
    return false;
}

/* Return image tag to display next to news
 * @param object $course the course to get the image for
 * @param string $alttext the alttext to use for this image
 * @return mixed bool | string - false if no image; HTML to display image if it does exist
 */
function format_qmultopics_getimage($course, $alttext = '') {
    global $CFG;

    $context = get_context_instance(CONTEXT_COURSE, $course->id);
    $section = get_course_section(0, $course->id);
    if (! $file = format_qmultopics_getimagefile($context, $section) ) {
        return false;
    }
    $path = '/' . $context->id . '/course/section/' . $section->id . $file->get_filepath() . $file->get_filename();
    $fullurl = file_encode_url($CFG->wwwroot . '/pluginfile.php', $path, false);
    // Prevent browser image caching when file changes
    if (strpos($fullurl, '?') === false) {
        $fullurl .= '?ts='.$file->get_timemodified();
    } else {
        $fullurl .= '&amp;ts='.$file->get_timemodified();
    }
    $mimetype = $file->get_mimetype();
    if (!in_array($mimetype, array('image/gif', 'image/jpeg', 'image/png'))) {  // It's not an image
        return '';
    }
    $tag = '<div class="static-img"><img src="' . $fullurl . '" alt="' . s($alttext) . '" '
            . 'height="' . FORMAT_QMULTOPICS_IMAGE_HEIGHT . '" width="' . FORMAT_QMULTOPICS_IMAGE_HEIGHT . '"/></div>';
    return $tag;
}

/*
 * Return string with latest news
 * @param object course
 */
function format_qmultopics_getnews($course) {
    global $CFG;

    require_once($CFG->dirroot . '/mod/forum/lib.php');   // We'll need this

    $text = '';

    if (!$forum = forum_get_course_forum($course->id, 'news')) {
        return '';
    }

    $modinfo = get_fast_modinfo($course);
    if (empty($modinfo->instances['forum'][$forum->id])) {
        return '';
    }
    $cm = $modinfo->instances['forum'][$forum->id];

    if (!$cm->uservisible) {
        return '';
    }

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);

    /// User must have perms to view discussions in that forum
    if (!has_capability('mod/forum:viewdiscussion', $context)) {
        return '';
    }

    /// First work out whether we can post to this group and if so, include a link
    $groupmode = groups_get_activity_groupmode($cm);
    $currentgroup = groups_get_activity_group($cm, true);

    /// Get all the recent discussions we're allowed to see

    if (!$discussions = forum_get_discussions($cm, 'p.modified DESC', true, $currentgroup, 1)) {
        $text .= '(' . get_string('nonews', 'forum') . ')';
    }

    /// Actually create the listing now

    $strftimerecent = get_string('strftimerecent');
    $strmore = get_string('more', 'forum');

    /// Accessibility: markup as a list.
    foreach ($discussions as $discussion) {

        $discussion->subject = $discussion->name;

        $discussion->subject = format_string($discussion->subject, true, $forum->course);

        $text .=
                '<span class="head clearfix">' .
                '<span class="info"><b><br />' . $discussion->subject . '</b><br />' .
                '<span class="info">' . strip_tags($discussion->message) . '</span><br />' .
                '<span class="name"><i>' . fullname($discussion) . '</i></span> ' .
                '<span class="date"><i>' . userdate($discussion->modified, $strftimerecent) . '</i></span>' .
                '</span>' .
                "\n";
    }

    //Links to older topics
    if (count($discussions)) {
        $text .=
                '<span class="newslink">' .
                '<a href="' . $CFG->wwwroot . '/mod/forum/view.php?f=' . $forum->id . '&amp;showall=1">' .
                get_string('morenews', 'format_qmultopics') .
                '</a>' .
                '</span>';
    }

    return $text;
}

/**
 * Stores optimised icon images in icon file area
 * Modified version of function process_new_icon in lib/gdlib.php
 *
 * @param $context
 * @param component
 * @param $itemid
 * @param $originalfile
 * @return success
 */
function format_qmultopics_process_new_icon($context, $section, $originalfile) {
    global $CFG;

    if (empty($CFG->gdversion)) {
        return false;
    }

    if (!is_file($originalfile)) {
        return false;
    }

    $imageinfo = GetImageSize($originalfile);

    if (empty($imageinfo)) {
        return false;
    }

    $image = new stdClass();
    $image->width = $imageinfo[0];
    $image->height = $imageinfo[1];
    $image->type = $imageinfo[2];

    switch ($image->type) {
        case IMAGETYPE_GIF:
            if (function_exists('ImageCreateFromGIF')) {
                $im = ImageCreateFromGIF($originalfile);
            } else {
                debugging('GIF not supported on this server');
                return false;
            }
            break;
        case IMAGETYPE_JPEG:
            if (function_exists('ImageCreateFromJPEG')) {
                $im = ImageCreateFromJPEG($originalfile);
            } else {
                debugging('JPEG not supported on this server');
                return false;
            }
            break;
        case IMAGETYPE_PNG:
            if (function_exists('ImageCreateFromPNG')) {
                $im = ImageCreateFromPNG($originalfile);
            } else {
                debugging('PNG not supported on this server');
                return false;
            }
            break;
        default:
            return false;
    }

    if (function_exists('ImagePng')) {
        $imagefnc = 'ImagePng';
        $imageext = '.png';
        $filters = PNG_NO_FILTER;
        $quality = 1;
    } else if (function_exists('ImageJpeg')) {
        $imagefnc = 'ImageJpeg';
        $imageext = '.jpg';
        $filters = null; // not used
        $quality = 90;
    } else {
        debugging('Jpeg and png not supported on this server, please fix server configuration');
        return false;
    }

    if (function_exists('ImageCreateTrueColor') and $CFG->gdversion >= 2) {
        $im1 = ImageCreateTrueColor(FORMAT_QMULTOPICS_IMAGE_HEIGHT, FORMAT_QMULTOPICS_IMAGE_HEIGHT);
        if ($image->type == IMAGETYPE_PNG and $imagefnc === 'ImagePng') {
            imagealphablending($im1, false);
            $color = imagecolorallocatealpha($im1, 0, 0, 0, 127);
            imagefill($im1, 0, 0, $color);
            imagesavealpha($im1, true);
        }
    } else {
        $im1 = ImageCreate(FORMAT_QMULTOPICS_IMAGE_HEIGHT, FORMAT_QMULTOPICS_IMAGE_HEIGHT);
    }

    $cx = $image->width / 2;
    $cy = $image->height / 2;

    if ($image->width < $image->height) {
        $half = floor($image->width / 2.0);
    } else {
        $half = floor($image->height / 2.0);
    }

    ImageCopyBicubic($im1, $im, 0, 0, $cx - $half, $cy - $half, FORMAT_QMULTOPICS_IMAGE_HEIGHT, FORMAT_QMULTOPICS_IMAGE_HEIGHT, $half * 2, $half * 2);

    ob_start();
    if (!$imagefnc($im1, NULL, $quality, $filters)) {
        // keep old icons
        ob_end_clean();
        return false;
    }
    $data = ob_get_clean();
    ImageDestroy($im1);
    // Cannot delete all area files, as that would delete any files attached via the HTML editor
    format_qmultopics_delete_icon($context, $section);
    $fs = get_file_storage();
    $icon = array('contextid' => $context->id, 'component' => 'course', 'filearea' => 'section', 'itemid' => $section->id, 'filepath' => '/');
    $icon['filename'] = FORMAT_QMULTOPICS_IMAGE_NAME . $imageext;
    $fs->create_file_from_string($icon, $data);
    return true;
}

function format_qmultopics_delete_icon($context, $section) {
    if ($file = format_qmultopics_getimagefile($context, $section)) {
        $file->delete();
    }
}